<!DOCTYPE html>
<html lang="{{ language }}">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{{ title }}</title>
    <link rel="icon" type="image/x-icon" href="/favicon.ico">
    <style>
        body {
            margin: 0;
            padding: 0;
            display: flex;
            align-items: center;
            justify-content: center;
            height: 100vh;
            background: linear-gradient(45deg, #47a8c9, #7a68a2, #f0b3a3);
            background-size: 400% 400%;
            animation: gradient 15s ease infinite;
        }

        @keyframes gradient {
            0% {
                background-position: 0% 50%;
            }

            50% {
                background-position: 100% 50%;
            }

            100% {
                background-position: 0% 50%;
            }
        }

        .login-form {
            background: white;
            backdrop-filter: blur(10px);
            border: none;
            border-radius: 10px;
            padding: 20px;
            width: 300px;
            text-align: center;
        }

        .login-form img {
            width: 100px;
            margin-bottom: 10px;
        }

        .login-form label {
            color: white;
            margin-right: 10px;
        }

        .login-form a {
            color: #007bff;
            text-decoration: none;
        }

        .login-form input[type="text"],
        .login-form input[type="password"] {
            width: 250px;
            height: 50px;
            padding: 0px;
            margin: 10px 0;
            border: 1px solid gray;
            border-radius: 5px;
            background: white;
            color: black;
            outline: none;
            font-size: 18px;
            text-align: center;
        }

        .login-form input[type="submit"] {
            width: 250px;
            height: 50px;
            padding: 0px;
            margin: 10px 0;
            border: none;
            border-radius: 5px;
            background: #007bff;
            color: white;
            cursor: pointer;
            font-size: 18px;
            text-align: center;
        }

        .links-info {
            position: absolute;
            bottom: 10px;
            left: 50%;
            transform: translateX(-50%);
            color: white;
            text-align: center;
        }
    </style>
</head>

<body>
    <form class="login-form" method="post">
        <p id="Prompt" style="color: black; font-size: 20px;">{{ input_prompt }}</p>
        <input name="username" type="text" placeholder="{{ input_username_placeholder }}" required><br>
        <input name="password" type="password" placeholder="{{ input_password_placeholder }}" required><br>
        <input type="submit" value="{{ input_submit_value }}"><br>
        <p id="errorPrompt" style="color: red; font-weight: bold;">{{ input_error_prompt }}</p>
    </form>
    <br>
    <div class="links-info">
        <p><a href="{{ url }}" style="font-weight: bold;">{{ powered_by }}</a></p>
    </div>
</body>

</html>